@@ -93,7 +93,7 @@ Some generic base classes and functions can be found in L{zim.utils}
 
 
 # Bunch of meta data, used at least in the about dialog
-__version__ = '0.72.0'
+__version__ = '0.73.0'
 __url__ = 'https://www.zim-wiki.org'
 __author__ = 'Jaap Karssenberg <jaap.karssenberg@gmail.com>'
 __copyright__ = 'Copyright 2008 - 2019 Jaap Karssenberg <jaap.karssenberg@gmail.com>'
@@ -165,8 +165,15 @@ try:
 		# We are running from a source dir - use the locale data included there
 		gettext.install('zim', _localedir, names=('_', 'gettext', 'ngettext'))
 	else:
-		# Hope the system knows where to find the data
-		gettext.install('zim', None, names=('_', 'gettext', 'ngettext'))
+		if os.getenv("TEXTDOMAINDIR"):
+			# TEXTDOMAINDIR is actually an official environment variable with
+			# gettext, but there's a catch: it's not being evaluated by the library
+			# itself, only by gettext's CLI tools. So we're repurposing a familiar
+			# name here instead of creating a variable name of our own.
+			gettext.install('zim', os.getenv("TEXTDOMAINDIR"), names=('_', 'gettext', 'ngettext'))
+		else:
+			# Hope the system knows where to find the data.
+			gettext.install('zim', None, names=('_', 'gettext', 'ngettext'))
 except:
 	logger.exception('Error loading translation')
 	trans = gettext.NullTranslations()
