@@ -62,7 +62,7 @@ def fix_line_end(text):
 
 def convert_space_to_tab(text, tabstop=4):
 	'''Convert spaces to tabs
-	@param text: the intput text
+	@param text: the input text
 	@param tabstop: the number of spaces to represent a tab
 	@returns: the fixed text
 	'''
@@ -286,7 +286,7 @@ class ParserError(Error):
 
 
 class Rule(object):
-	'''Class that defines a sigle parser rule. Typically used
+	'''Class that defines a single parser rule. Typically used
 	to define a regex pattern for one specific wiki format string
 	and the processing to be done when this formatting is encountered
 	in the text.
@@ -316,7 +316,10 @@ class Rule(object):
 		assert tag is not None or process is not None, 'Need at least a tag or a process method'
 		self._re = None
 		self.tag = tag
-		self.pattern = pattern
+		if isinstance(pattern, str):
+			self.pattern = pattern
+		else:
+			self.pattern = pattern.pattern # Assume compiled regular expression
 		self.descent = descent
 		self.process = process or self._process
 
@@ -333,7 +336,7 @@ class Rule(object):
 		# default action for matched text
 		if self.descent:
 			builder.start(self.tag)
-			self.descent(builder, *text)
+			self.descent(builder, text)
 			builder.end(self.tag)
 		else:
 			builder.append(self.tag, None, text)
@@ -344,7 +347,7 @@ class Parser(object):
 	compile the patterns of various rules into a single regex and
 	based on the match call the correct rules for processing.
 
-	@ivar rules: list with L{Rule} objects, can be modified untill the
+	@ivar rules: list with L{Rule} objects, can be modified until the
 	parser is used for the first time for parsing (the attribute
 	becomes a tuple afterwards)
 	@ivar process_unmatched: function (or object) to process un-matched
@@ -388,7 +391,7 @@ class Parser(object):
 	def __call__(self, builder, text):
 		'''Each parser object is callable so it can be used as a
 		processing function in any other parser object. This method
-		parses the given text and calls the apropriate methods of the
+		parses the given text and calls the appropriate methods of the
 		L{Builder} object to construct the parse results.
 
 		@param builder: a L{Builder} object
@@ -403,12 +406,13 @@ class Parser(object):
 				r"(?P<rule%i>%s)" % (i, r.pattern)
 					for i, r in enumerate(self.rules)
 			])
-			#~ print('PATTERN:\n', pattern.replace(')|(', ')\t|\n('), '\n...')
+			#print('PATTERN:\n', pattern.replace(')|(', ')\t|\n('), '\n...')
 			self._re = re.compile(pattern, re.U | re.M | re.X)
 
 		iter = 0
 		end = len(text)
-		for match in self._re.finditer(text):
+		match = self._re.search(text, iter)
+		while match:
 			mstart, mend = match.span()
 			if mstart > iter:
 				try:
@@ -422,13 +426,14 @@ class Parser(object):
 			if len(groups) > 1:
 				groups.pop(0) # get rid of named outer group if inner groups are defined
 
+			self._backup_iter = 0
 			try:
 				self.rules[i].process(builder, *groups)
 			except Exception as error:
 				self._raise_exception(error, text, mstart, mend, builder, self.rules[i])
 
-			iter = mend
-
+			iter = mend - self._backup_iter
+			match = self._re.search(text, iter)
 		else:
 			# no more matches
 			if iter < end:
@@ -439,6 +444,9 @@ class Parser(object):
 
 	parse = __call__
 
+	def backup_parser_offset(self, i):
+		self._backup_iter += i
+
 	@staticmethod
 	def _raise_exception(error, text, start, end, builder, rule=None):
 		# Add parser state, line count etc. to error, then re-raise
@@ -472,7 +480,7 @@ def get_line_count(text, offset):
 	@returns: a 2-tuple of the line and column that corresponds to this
 	offset
 	'''
-	# line numbers start counting at 1, colums at 0
+	# line numbers start counting at 1, columns at 0
 	if offset == 0:
 		return 1, 0
 	slice = text[:offset]
