@@ -13,19 +13,21 @@ import logging
 from functools import partial
 
 from zim.notebook import Path
-from zim.notebook.index.pages import PagesTreeModelMixin, PageIndexRecord, IndexNotFoundError
+from zim.notebook.index.base import TreeModelMixinBase
+from zim.notebook.index.pages import PagesTreeModelMixin, PageIndexRecord, IndexNotFoundError, IS_PAGE
 
 from zim.plugins import PluginClass
 from zim.actions import PRIMARY_MODIFIER_MASK
 
 from zim.gui.pageview import PageViewExtension
 from zim.gui.widgets import BrowserTreeView, ScrolledWindow, \
-	populate_popup_add_separator, encode_markup_text, ErrorDialog, \
+	encode_markup_text, ErrorDialog, \
 	WindowSidePaneWidget, LEFT_PANE, PANE_POSITIONS
 from zim.gui.clipboard import Clipboard, pack_urilist, unpack_urilist, \
 	INTERNAL_PAGELIST_TARGET_NAME, INTERNAL_PAGELIST_TARGET
-from zim.gui.uiactions import UIActions, PAGE_ACTIONS
+from zim.gui.uiactions import UIActions, PAGE_EDIT_ACTIONS, PAGE_ROOT_ACTIONS
 
+import zim.gui.clipboard
 
 logger = logging.getLogger('zim.gui.pageindex')
 
@@ -58,6 +60,10 @@ This plugin adds the page index pane to the main window.
 		# key, type, label, default
 		('pane', 'choice', _('Position in the window'), LEFT_PANE, PANE_POSITIONS),
 			# T: preferences option
+		('autoexpand', 'bool', _('Automatically expand sections on open page'), True),
+			# T: preferences option
+		('autocollapse', 'bool', _('Automatically collapse sections on close page'), True),
+			# T: preferences option
 	)
 
 
@@ -70,6 +76,7 @@ class PageIndexPageViewExtension(PageViewExtension):
 		self.treeview = PageTreeView(pageview.notebook, self.navigation)
 		self.treeview.set_model(model)
 		self.widget = PageIndexWidget(self.treeview)
+		self._autoexpanded = None
 
 		# Connect to ui signals
 		#window.connect('start-index-update', lambda o: self.disconnect_model())
@@ -85,7 +92,20 @@ class PageIndexPageViewExtension(PageViewExtension):
 
 	def on_page_changed(self, pageview, page):
 		treepath = self.treeview.set_current_page(page, vivificate=True)
-		if treepath:
+
+		if self._autoexpanded and self.plugin.preferences['autocollapse']:
+			ref1, ref2 = self._autoexpanded
+			if ref1.valid() and (ref2 is None or ref2.valid()):
+				prev_treepath = ref1.get_path()
+				prev_expanded_path = ref2.get_path() if ref2 else None
+				self.treeview.restore_expanded_path(prev_treepath, prev_expanded_path)
+
+		if treepath and self.plugin.preferences['autoexpand']:
+			expanded_path = self.treeview.get_expanded_path(treepath)
+			model = self.treeview.get_model()
+			ref1 = Gtk.TreeRowReference(model, treepath)
+			ref2 = Gtk.TreeRowReference(model, expanded_path) if expanded_path else None
+			self._autoexpanded = (ref1, ref2)
 			self.treeview.select_treepath(treepath)
 
 	def disconnect_model(self):
@@ -102,6 +122,7 @@ class PageIndexPageViewExtension(PageViewExtension):
 		reloading the index to get rid of out-of-sync model errors
 		without need to close the app first.
 		'''
+		self.treeview.disconnect_index()
 		model = PageTreeStore(self.pageview.notebook.index)
 		self.treeview.set_model(model)
 
@@ -363,6 +384,7 @@ class PageTreeView(BrowserTreeView):
 		self.navigation = navigation
 
 		column = Gtk.TreeViewColumn('_pages_')
+		column.set_expand(True)
 		self.append_column(column)
 
 		cr1 = Gtk.CellRendererText()
@@ -371,6 +393,9 @@ class PageTreeView(BrowserTreeView):
 		column.set_attributes(cr1, text=NAME_COL,
 			style=STYLE_COL, sensitive=EXISTS_COL, weight=WEIGHT_COL)
 
+		column = Gtk.TreeViewColumn('_n_items_')
+		self.append_column(column)
+
 		cr2 = self.get_cell_renderer_number_of_items()
 		column.pack_start(cr2, False)
 		column.set_attributes(cr2, text=N_CHILD_COL, weight=WEIGHT_COL)
@@ -402,14 +427,18 @@ class PageTreeView(BrowserTreeView):
 		model = self.get_model()
 		if isinstance(model, Gtk.TreeModelFilter):
 			model = model.get_model() # get childmodel
-		model.teardown()
+
+		if isinstance(model, TreeModelMixinBase):
+			model.teardown()
 
 	def do_row_activated(self, treepath, column):
 		model = self.get_model()
-		iter = model.get_iter(treepath)
-		path = model.get_indexpath(iter)
-		if path:
-			self.emit('page-activated', path)
+		treeiter = model.get_iter(treepath)
+		mytreeiter = model.get_user_data(treeiter)
+		if mytreeiter.hint == IS_PAGE:
+			path = model.get_indexpath(treeiter)
+			if path:
+				self.emit('page-activated', path)
 
 	def do_page_activated(self, path):
 		self.navigation.open_page(path)
@@ -438,30 +467,30 @@ class PageTreeView(BrowserTreeView):
 			or BrowserTreeView.do_key_press_event(self, event)
 
 	def do_initialize_popup(self, menu):
-		# TODO get path first and determine what menu options are valid
-		path = self.get_selected_path() or Path(':')
+		model, treeiter = self.get_selection().get_selected()
+		if treeiter is None:
+			popup_name, path = PAGE_ROOT_ACTIONS, None
+		else:
+			mytreeiter = model.get_user_data(treeiter)
+			if mytreeiter.hint == IS_PAGE:
+				popup_name = PAGE_EDIT_ACTIONS
+				path = self.get_selected_path()
+			else:
+				popup_name, path = None, None
 
-		uiactions = UIActions(
-			self,
-			self.notebook,
-			path,
-			self.navigation,
-		)
-		uiactions.populate_menu_with_actions(PAGE_ACTIONS, menu)
-
-		populate_popup_add_separator(menu)
-		item = Gtk.MenuItem.new_with_mnemonic(_('_Copy')) # T: menu label
-		item.connect('activate', lambda o: self.do_copy())
-		menu.append(item)
-		menu.show_all()
+		if popup_name:
+			uiactions = UIActions(
+				self,
+				self.notebook,
+				path,
+				self.navigation,
+			)
+			uiactions.populate_menu_with_actions(popup_name, menu)
 
+		sep = Gtk.SeparatorMenuItem()
+		menu.append(sep)
 		self.populate_popup_expand_collapse(menu)
-
-	def do_copy(self):
-		#~ print('!! copy location')
-		page = self.get_selected_path()
-		if page:
-			Clipboard.set_pagelink(self.notebook, page)
+		menu.show_all()
 
 	def do_drag_data_get(self, dragcontext, selectiondata, info, time):
 		assert selectiondata.get_target().name() == INTERNAL_PAGELIST_TARGET_NAME
@@ -470,10 +499,16 @@ class PageTreeView(BrowserTreeView):
 		logger.debug('Drag data requested, we have internal path "%s"', path.name)
 		data = pack_urilist((path.name,))
 		selectiondata.set(selectiondata.get_target(), 8, data)
+		zim.gui.clipboard._internal_selection_data = data # HACK issue #390
 
 	def do_drag_data_received(self, dragcontext, x, y, selectiondata, info, time):
 		assert selectiondata.get_target().name() == INTERNAL_PAGELIST_TARGET_NAME
-		names = unpack_urilist(selectiondata.get_data())
+		data = selectiondata.get_data()
+		if data is None:
+			data = zim.gui.clipboard._internal_selection_data # HACK issue #390
+			zim.gui.clipboard._internal_selection_data = None
+
+		names = unpack_urilist(data)
 		assert len(names) == 1
 		source = Path(names[0])
 
@@ -555,3 +590,31 @@ class PageTreeView(BrowserTreeView):
 			return None
 		else:
 			return model.get_indexpath(iter)
+
+	def get_expanded_path(self, path):
+		'''Return the lowest expanded path towards C{path}'''
+		path = path.copy()
+		while path and not self.row_expanded(path):
+			if path.get_depth() == 1:
+				path = None
+			else:
+				path.up()
+		return path
+
+	def restore_expanded_path(self, path, expanded_path):
+		'''Collaps path between C{path} and C{expanded_path}'''
+		path = path.copy()
+		if expanded_path is None:
+			while path:
+				self.collapse_row(path)
+				if path.get_depth() == 1:
+					path = None
+				else:
+					path.up()
+		else:
+			while path.compare(expanded_path) == 1: # else equal or deeper
+				self.collapse_row(path)
+				if path.get_depth() == 1:
+					path = None
+				else:
+					path.up()
