@@ -62,12 +62,14 @@ class TaskListWidget(Gtk.VBox, TaskListWidgetMixin, WindowSidePaneWidget):
 
 	title = _('Tasks') # T: tab label for side pane
 
-	def __init__(self, tasksview, opener, properties, uistate):
+	def __init__(self, tasksview, opener, properties, with_due, uistate):
 		GObject.GObject.__init__(self)
 		self.uistate = uistate
 		self.uistate.setdefault('only_show_act', False)
 		self.uistate.setdefault('show_flatlist', False)
 
+		column_layout=TaskListTreeView.COMPACT_COLUMN_LAYOUT_WITH_DUE \
+			if with_due else TaskListTreeView.COMPACT_COLUMN_LAYOUT
 		self.task_list = TaskListTreeView(
 			tasksview, opener,
 			_parse_task_labels(properties['labels']),
@@ -75,7 +77,7 @@ class TaskListWidget(Gtk.VBox, TaskListWidgetMixin, WindowSidePaneWidget):
 			filter_actionable=self.uistate['only_show_act'],
 			tag_by_page=properties['tag_by_page'],
 			use_workweek=properties['use_workweek'],
-			compact=True,
+			column_layout=column_layout,
 			flatlist=self.uistate['show_flatlist'],
 		)
 		self.task_list.connect('populate-popup', self.on_populate_popup)
@@ -362,10 +364,6 @@ def days_to_str(days):
 
 class TaskListTreeView(BrowserTreeView):
 
-	# TODO "compact" view should be subclass
-	#  Base <|-- View
-	#  Base <|-- CompactView
-	#
 	# idem for flat list vs tree
 
 	VIS_COL = 0 # visible
@@ -380,12 +378,16 @@ class TaskListTreeView(BrowserTreeView):
 	PRIO_SORT_COL = 9
 	PRIO_SORT_LABEL_COL = 10
 
+	RICH_COLUMN_LAYOUT = 11
+	COMPACT_COLUMN_LAYOUT = 12
+	COMPACT_COLUMN_LAYOUT_WITH_DUE = 13
+
 	def __init__(self,
 		tasksview, opener,
 		task_labels,
 		nonactionable_tags=(),
 		filter_actionable=False, tag_by_page=False, use_workweek=False,
-		compact=False, flatlist=False,
+		column_layout=RICH_COLUMN_LAYOUT, flatlist=False,
 		sort_column=PRIO_COL, sort_order=Gtk.SortType.DESCENDING
 	):
 		self.real_model = Gtk.TreeStore(bool, bool, int, str, str, object, str, str, int, int, str)
@@ -435,7 +437,7 @@ class TaskListTreeView(BrowserTreeView):
 		column.set_resizable(True)
 		column.set_sort_column_id(self.DESC_COL)
 		column.set_expand(True)
-		if compact:
+		if column_layout != self.RICH_COLUMN_LAYOUT:
 			column.set_min_width(100)
 		else:
 			column.set_min_width(300) # don't let this column get too small
@@ -479,7 +481,7 @@ class TaskListTreeView(BrowserTreeView):
 					color = None
 			cell.set_property('cell-background', color)
 
-		if not compact:
+		if column_layout != self.COMPACT_COLUMN_LAYOUT:
 			cell_renderer = Gtk.CellRendererText()
 			column = Gtk.TreeViewColumn(_('Date'), cell_renderer)
 				# T: Column header Task List dialog
@@ -488,7 +490,7 @@ class TaskListTreeView(BrowserTreeView):
 			self.append_column(column)
 
 		# Rendering for page name column
-		if not compact:
+		if column_layout == self.RICH_COLUMN_LAYOUT:
 			cell_renderer = Gtk.CellRendererText()
 			column = Gtk.TreeViewColumn(_('Page'), cell_renderer, text=self.PAGE_COL)
 					# T: Column header Task List dialog
@@ -614,7 +616,7 @@ class TaskListTreeView(BrowserTreeView):
 				prio_sort_label = '!' * min(row['prio'], 3)
 
 			# Format description
-			desc = _date_re.sub('', row['description'], count=1)
+			desc = _date_re.sub('', row['description'])
 			desc = re.sub('\s*!+\s*', ' ', desc) # get rid of exclamation marks
 			desc = encode_markup_text(desc)
 			if actionable:
